@@ -74,6 +74,8 @@ GdkPixbuf *openna;
 GdkPixbuf *fedora;
 GdkPixbuf *conectiva;
 GdkPixbuf *black_panther;
+GdkPixbuf *vine;
+GdkPixbuf *specifix;
 
 enum {
 	BOGUS,
@@ -437,6 +439,8 @@ gst_tool_create_distro_images (void)
 	fedora = gdk_pixbuf_new_from_file (PIXMAPS_DIR "/fedora.png", NULL);
 	conectiva = gdk_pixbuf_new_from_file (PIXMAPS_DIR "/conectiva.png", NULL);
 	black_panther = gdk_pixbuf_new_from_file (PIXMAPS_DIR "/black_panther.png", NULL);
+	vine = gdk_pixbuf_new_from_file (PIXMAPS_DIR "/vine.png", NULL);
+	specifix = gdk_pixbuf_new_from_file (PIXMAPS_DIR "/specifix.png", NULL);
 }
 
 static void
