@@ -48,85 +48,53 @@ service_get_description (xmlNodePtr service)
 	return description;
 }
 
-static gint
+static gchar*
 get_current_runlevel (GstTool *tool)
 {
-	GtkWidget *option_menu, *menu, *selected_option;
-	gint runlevel;
-	
-	option_menu = gst_dialog_get_widget (tool->main_dialog, "runlevels_menu");
-	menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (option_menu));
-	selected_option = gtk_menu_get_active (GTK_MENU (menu));
+	GtkWidget    *combo;
+	GtkTreeModel *model;
+	GtkTreeIter   iter;
+	gchar        *str = NULL;
 
-	runlevel = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (selected_option), "runlevel"));
+	combo = gst_dialog_get_widget (tool->main_dialog, "runlevels_menu");
+	model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
 
-	return runlevel;
-}
+	if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combo), &iter))
+		gtk_tree_model_get (model, &iter, 1, &str, -1);
 
-static gboolean
-current_runlevel_is_default (GstTool *tool)
-{
-	GtkWidget *option_menu, *menu, *selected_option;
-	gboolean default_runlevel;
-	
-	option_menu = gst_dialog_get_widget (tool->main_dialog, "runlevels_menu");
-	menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (option_menu));
-	selected_option = gtk_menu_get_active (GTK_MENU (menu));
-
-	default_runlevel = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (selected_option), "default"));
-
-	return default_runlevel;
-}
-
-/* if we are changing the same runlevel than the default,
- * start or stop the service */
-static void
-run_service (GstTool *tool, xmlNodePtr service, gchar *runlevel, gchar *action)
-{
-	gchar *script;
-	
-	if (current_runlevel_is_default (tool)) {
-		script = gst_xml_get_child_content (service, "script");
-		gst_tool_run_get_directive (tool, NULL, "throw_service", script, action, NULL);
-
-		g_free (script);
-	}
+	return str;
 }
 
 static void
-toggle_service (GstTool *tool, xmlNodePtr service, gint runlevel, gboolean status)
+toggle_service (GstTool *tool, xmlNodePtr service, gchar* runlevel, gboolean status)
 {
 	xmlNodePtr runlevels = gst_xml_element_find_first (service, "runlevels");
 	xmlNodePtr node;
-	gchar *buf, *r, *action;
+	gchar *r, *action;
 	gboolean found = FALSE;
 
-
-
-	buf = g_strdup_printf ("%i", runlevel);
-
 	if (status == TRUE)
 		action = g_strdup_printf ("start");
 	else
 		action = g_strdup_printf ("stop");
 
-	if (runlevels == NULL) {
-		runlevels = gst_xml_element_add (node, "runlevels");
-	}
-	
-	/* if the node already exists, put its action to "start" or "stop" */
-	for (node = gst_xml_element_find_first (runlevels, "runlevel");
-	     node != NULL;
-	     node = gst_xml_element_find_next (node, "runlevel"))
-	{
-		r = gst_xml_get_child_content (node, "number");
-		
-		if (strcmp (r, buf) == 0) {
-			gst_xml_set_child_content (node, "action", action);
-			found = TRUE;
-		}
+	if (runlevels == NULL)
+		runlevels = gst_xml_element_add (service, "runlevels");
+	else {
+		/* if the node already exists, put its action to "start" or "stop" */
+		for (node = gst_xml_element_find_first (runlevels, "runlevel");
+		     node != NULL;
+		     node = gst_xml_element_find_next (node, "runlevel"))
+		{
+			r = gst_xml_get_child_content (node, "number");
 
-		g_free (r);
+			if (strcmp (r, runlevel) == 0) {
+				gst_xml_set_child_content (node, "action", action);
+				found = TRUE;
+			}
+
+			g_free (r);
+		}
 	}
 
 	/* if the node hasn't been found, create it */
@@ -134,16 +102,41 @@ toggle_service (GstTool *tool, xmlNodePtr service, gint runlevel, gboolean statu
 		node = gst_xml_element_add (runlevels, "runlevel");
 		gst_xml_element_add (node, "number");
 		gst_xml_element_add (node, "action");
-		gst_xml_set_child_content (node, "number", buf);
+		gst_xml_set_child_content (node, "number", runlevel);
 		gst_xml_set_child_content (node, "action", action);
 	}
-
-/*	run_service (tool, service, buf, action); */
-
-	g_free (buf);
-
 }
 
+void
+change_runlevel (gchar *runlevel)
+{
+	xmlNodePtr root = gst_xml_doc_get_root (tool->config);
+
+	table_clear ();
+	table_populate (root, runlevel);
+}
+
+/* check the first service, if it doesn't have priority,
+ * hide the "order by startup sequence" button */
+void
+hide_sequence_ordering_toggle_button (xmlNodePtr root)
+{
+	GtkWidget *widget = gst_dialog_get_widget (tool->main_dialog, "sequence_ordering");
+	xmlNodePtr services, service, priority;
+
+	services = gst_xml_element_find_first (root, "services");
+	g_return_if_fail (services != NULL);
+
+	service = gst_xml_element_find_first (services, "service");
+	g_return_if_fail (service != NULL);
+
+	priority = gst_xml_element_find_first (service, "priority");
+
+	if (!priority)
+		gtk_widget_hide (widget);
+	else
+		gtk_widget_show (widget);
+}
 
 /* callbacks */
 static void
@@ -154,16 +147,6 @@ callbacks_set_buttons_sensitive (gboolean enabled)
 	gtk_widget_set_sensitive (settings_button, enabled);
 }
 
-static void
-callbacks_description_changed (xmlNodePtr service)
-{
-	gchar *description = service_get_description (service);
-	GtkWidget *label = gst_dialog_get_widget (tool->main_dialog, "description_label");
-
-	gtk_label_set_text (GTK_LABEL (label), description);
-	g_free (description);
-}
-
 void
 on_services_table_select_row (GtkTreeSelection *selection, gpointer data)
 {
@@ -172,18 +155,10 @@ on_services_table_select_row (GtkTreeSelection *selection, gpointer data)
 	GtkTreeIter iter;
 	xmlNodePtr service;
 
-	if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
+	if (gtk_tree_selection_get_selected (selection, &model, &iter))
 		callbacks_set_buttons_sensitive (TRUE);
-
-		/* get the xmlNodePtr */
-		gtk_tree_model_get (model, &iter, COL_POINTER, &service, -1);
-
-		/* Change the description label */
-		callbacks_description_changed (service);
-
-	} else {
+	else
 		callbacks_set_buttons_sensitive (FALSE);
-	}
 }
 
 void
@@ -212,6 +187,9 @@ on_service_priority_changed (GtkWidget *spin_button, gpointer data)
 
 	gst_xml_set_child_content (service, "priority", value);
 
+	gtk_tree_store_set (GTK_TREE_STORE (model), &iter, COL_PRIORITY, val, -1);
+	gtk_tree_view_scroll_to_cell (runlevel_table, path, NULL, TRUE, 0.5, 0.5);
+
 	gst_dialog_modify (tool->main_dialog);
 
 	g_free (value);
@@ -287,7 +265,7 @@ on_settings_button_clicked (GtkWidget *button, gpointer data)
 	/* get the priority, if it doesn't exist, we simply hide the spinbutton */
 	p = gst_xml_get_child_content (service, "priority");
 
-	if (p == NULL) {
+	if (!p) {
 		gtk_widget_hide (service_priority);
 		gtk_widget_hide (service_priority_label);
 	} else {
@@ -316,6 +294,8 @@ on_settings_button_clicked (GtkWidget *button, gpointer data)
 
 	dialog_service_get_status (script);
 
+	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (tool->main_dialog));
+
 	response = gtk_dialog_run (GTK_DIALOG (dialog));
 
 	if (response == RESPONSE_STOP_SERVICE)
@@ -334,11 +314,10 @@ on_settings_button_clicked (GtkWidget *button, gpointer data)
 void
 on_runlevel_changed (GtkWidget *widget, gpointer data)
 {
-	gint runlevel = GPOINTER_TO_INT (data);
-	xmlNodePtr root = gst_xml_doc_get_root (tool->config);
+	gchar      *runlevel;
 
-	table_clear ();
-	table_populate (root, runlevel);
+	runlevel  = get_current_runlevel (tool);
+	change_runlevel (runlevel);
 }
 
 void
@@ -347,7 +326,7 @@ on_service_toggled (GtkWidget *widget, gchar *path_str, gpointer data)
 	gboolean value = gtk_cell_renderer_toggle_get_active (GTK_CELL_RENDERER_TOGGLE (widget));
 	gboolean new_value = !value;
 	GstTool *tool = GST_TOOL (data);
-	gint runlevel = get_current_runlevel (tool);
+	gchar *runlevel = get_current_runlevel (tool);
 	GtkTreeView *runlevel_table = GTK_TREE_VIEW (gst_dialog_get_widget (tool->main_dialog, "runlevel_table"));
 	GtkTreeModel *model = gtk_tree_view_get_model (runlevel_table);
 	GtkTreePath *path = gtk_tree_path_new_from_string (path_str);
@@ -380,18 +359,32 @@ on_popup_settings_activate (gpointer callback_data, guint action, GtkWidget *wid
 	on_settings_button_clicked (widget, callback_data);
 }
 
+static void
+do_popup_menu (GtkWidget *popup, GdkEventButton *event)
+{
+	gint button, event_time;
+
+	if (!popup)
+		return;
+
+	if (event) {
+		button     = event->button;
+		event_time = event->time;
+	} else {
+		button     = 0;
+		event_time = gtk_get_current_event_time ();
+	}
+
+	gtk_menu_popup (GTK_MENU (popup), NULL, NULL, NULL, NULL,
+			button, event_time);
+}
+
 gboolean
-on_table_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer data)
+on_table_button_press_event (GtkWidget *widget, GdkEventButton *event, GtkWidget *popup)
 {
 	GtkTreePath *path;
-	GtkItemFactory *item_factory;
 	GtkTreeView *treeview = GTK_TREE_VIEW (widget);
 
-	item_factory = (GtkItemFactory *) data;
-
-	if (gst_dialog_get_complexity (tool->main_dialog) == GST_DIALOG_BASIC)
-		return;
-
 	if (event->button == 3) {
 		gtk_widget_grab_focus (widget);
 
@@ -400,8 +393,7 @@ on_table_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer
 			gtk_tree_selection_unselect_all (gtk_tree_view_get_selection (treeview));
 			gtk_tree_selection_select_path (gtk_tree_view_get_selection (treeview), path);
 
-			gtk_item_factory_popup (item_factory, event->x_root, event->y_root,
-						event->button, event->time);
+			do_popup_menu (popup, event);
 		}
 
 		return TRUE;
@@ -409,3 +401,28 @@ on_table_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer
 
 	return FALSE;
 }
+
+gboolean
+on_table_popup_menu (GtkWidget *widget, GtkWidget *popup)
+{
+	do_popup_menu (popup, NULL);
+	return TRUE;
+}
+
+void
+on_sequence_ordering_changed (GtkWidget *widget, gpointer data)
+{
+	GtkTreeView *treeview = GTK_TREE_VIEW (gst_dialog_get_widget (tool->main_dialog, "runlevel_table"));
+	GtkTreeViewColumn *services_column = gtk_tree_view_get_column (treeview, COL_SERVICE);
+	GtkTreeViewColumn *priority_column = gtk_tree_view_get_column (treeview, COL_PRIORITY);
+	gboolean active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
+
+	if (active) {
+		/* FIXME: is there any other way? */
+		gtk_tree_view_column_set_visible (priority_column, TRUE);
+		gtk_tree_view_column_clicked (priority_column);
+		gtk_tree_view_column_set_visible (priority_column, FALSE);
+	} else {
+		gtk_tree_view_column_clicked (services_column);
+	}
+}
